/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *  Goo
 *
 *  Copyright (C) 2004 Free Software Foundation, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 */

#ifndef GOO_STOCK_H
#define GOO_STOCK_H

#define GOO_STOCK_PLAY        "media-playback-start-symbolic"
#define GOO_STOCK_PAUSE       "media-playback-pause-symbolic"
#define GOO_STOCK_STOP        "media-playback-stop-symbolic"
#define GOO_STOCK_NEXT        "media-skip-forward-symbolic"
#define GOO_STOCK_PREV        "media-skip-backward-symbolic"
#define GOO_STOCK_EJECT       "media-eject-symbolic"
#define GOO_STOCK_EXTRACT     "document-save-symbolic"
#define GOO_STOCK_RESET       "goo-reset"
#define GOO_STOCK_NO_DISC     "media-optical-symbolic"
#define GOO_STOCK_DATA_DISC   "drive-harddisk-symbolic"
#define GOO_STOCK_AUDIO_CD    "media-optical-symbolic"
#define GOO_STOCK_VOLUME_MAX  "audio-volume-high-symbolic"
#define GOO_STOCK_VOLUME_MED  "audio-volume-medium-symbolic"
#define GOO_STOCK_VOLUME_MIN  "audio-volume-low-symbolic"
#define GOO_STOCK_VOLUME_ZERO "audio-volume-muted-symbolic"


void goo_stock_init (void);

#endif /* GOO_STOCK_H */
